'use strict';

// ** Override DEBUG environment variable if it has no value to show info messages by default
if (process.env.hasOwnProperty("DEBUG") === false) {
    process.env["DEBUG"] = "*:info,*.warn";
}

// ** Dependencies
const path = require('path');
const debug = require('debug');
const functions = require('./functions');

/**
 * Returns the name of the logger or uses the filename of who required the file
 * @param name
 * @returns {*}
 */
function getLoggerName(name) {

    if (!name) {
        // ** Load the options name from the callsite
        const caller = functions.getCallerInfo();
        const filename = caller.filename;

        // ** Don't include anything beyond the nodus-server directory

        name = path.relative(process.cwd(), filename);
        // name = trimPath(filename, 'nodus-server');
        // name = trimPath(name, 'nodus-framework');
        // name = trimPath(name, 'nodus-run.sh');
    }

    return name;
}

function InfoLogger (name) {
    const logger = debug(`${name}:info`);
    logger.log = console.log.bind(console);

    return logger;
}

function DebugLogger (name) {
    const logger = debug(`${name}:debug`); // Write to STDERR by default
    return logger;
}

function TraceLogger(name) {
    const logger = debug(`${name}:trace`);
    logger.log = console.trace.bind(console);

    return logger;
}

function ErrorLogger(name) {
    //const logger = debug(`${name}:info`);;
    //logger.log = console.error.bind(console);
    const logger = console.error;

    return logger;
}

function WarnLogger(name) {
    const logger = debug(name);
    logger.log = console.warn.bind(console);

    return logger;
}

/**
 * Create a new instance of a logger
 * @param name - The name of the logger.  Defaults to the filename of the required file
 * @returns {*}
 */
function createLogger(name) {

    // TODO: Support building from app.json services and other program metadata
    const logName = getLoggerName(name);

    // Return a new debug logger
    return {
        info: InfoLogger(logName),
        debug: DebugLogger(logName),
        trace: TraceLogger(logName),
        error: ErrorLogger(logName),
        warn: WarnLogger(logName)
    };
}

// ** Exports
module.exports = createLogger;
module.exports.createLogger = createLogger;